@@ -26,7 +26,7 @@ import six
 
 try:
     import salt.utils.path
-except ImportError:
+except ImportError:  # pragma: no cover
     # We need salt to test salt with saltfactories, and, when pytest is rewriting modules for proper assertion
     # reporting, we still haven't had a chance to inject the salt path into sys.modules, so we'll hit this
     # import error, but its safe to pass
@@ -57,6 +57,11 @@ class Popen(subprocess.Popen):
         self.__stderr = stderr
         compat.weakref.finalize(self, stdout.close)
         compat.weakref.finalize(self, stderr.close)
+        if six.PY2:
+            # Under Py2, subprocess.Popen doesn't store the command line passed under
+            # it's args attribute.
+            # Let's do it ourselves
+            self.args = args[0]
 
     def communicate(self, input=None):  # pylint: disable=arguments-differ
         super(Popen, self).communicate(input)
@@ -82,7 +87,7 @@ class Popen(subprocess.Popen):
         return stdout, stderr
 
 
-class ProcessResult(namedtuple("ProcessResult", ("exitcode", "stdout", "stderr"))):
+class ProcessResult(namedtuple("ProcessResult", ("exitcode", "stdout", "stderr", "cmdline"))):
     """
     This class serves the purpose of having a common result class which will hold the
     resulting data from a subprocess command.
@@ -90,18 +95,19 @@ class ProcessResult(namedtuple("ProcessResult", ("exitcode", "stdout", "stderr")
 
     __slots__ = ()
 
-    def __new__(cls, exitcode, stdout, stderr):
+    def __new__(cls, exitcode, stdout, stderr, cmdline=None):
         if not isinstance(exitcode, int):
             raise ValueError("'exitcode' needs to be an integer, not '{}'".format(type(exitcode)))
-        return super(ProcessResult, cls).__new__(cls, exitcode, stdout, stderr)
+        return super(ProcessResult, cls).__new__(cls, exitcode, stdout, stderr, cmdline=cmdline)
 
     # These are copied from the namedtuple verbose output in order to quiet down PyLint
-    exitcode = property(itemgetter(0), doc="ShellResult exit code property")
-    stdout = property(itemgetter(1), doc="ShellResult stdout property")
-    stderr = property(itemgetter(2), doc="ShellResult stderr property")
+    exitcode = property(itemgetter(0), doc="ProcessResult exit code property")
+    stdout = property(itemgetter(1), doc="ProcessResult stdout property")
+    stderr = property(itemgetter(2), doc="ProcessResult stderr property")
+    cmdline = property(itemgetter(3), doc="ProcessResult cmdline property")
 
 
-class ShellResult(namedtuple("ShellResult", ("exitcode", "stdout", "stderr", "json"))):
+class ShellResult(namedtuple("ShellResult", ("exitcode", "stdout", "stderr", "json", "cmdline"))):
     """
     This class serves the purpose of having a common result class which will hold the
     resulting data from a subprocess command.
@@ -109,16 +115,19 @@ class ShellResult(namedtuple("ShellResult", ("exitcode", "stdout", "stderr", "js
 
     __slots__ = ()
 
-    def __new__(cls, exitcode, stdout, stderr, json):
+    def __new__(cls, exitcode, stdout, stderr, json=None, cmdline=None):
         if not isinstance(exitcode, int):
             raise ValueError("'exitcode' needs to be an integer, not '{}'".format(type(exitcode)))
-        return super(ShellResult, cls).__new__(cls, exitcode, stdout, stderr, json)
+        return super(ShellResult, cls).__new__(
+            cls, exitcode, stdout, stderr, json=json, cmdline=cmdline
+        )
 
     # These are copied from the namedtuple verbose output in order to quiet down PyLint
     exitcode = property(itemgetter(0), doc="ShellResult exit code property")
     stdout = property(itemgetter(1), doc="ShellResult stdout property")
     stderr = property(itemgetter(2), doc="ShellResult stderr property")
     json = property(itemgetter(3), doc="ShellResult stdout JSON decoded, when parseable.")
+    cmdline = property(itemgetter(4), doc="ShellResult cmdline property")
 
     def __eq__(self, other):
         """
@@ -257,7 +266,9 @@ class FactoryProcess(object):
                     )
                 log_message += "\n"
             log.info(log_message)
-            self._terminal_result = ProcessResult(self._terminal.returncode, stdout, stderr)
+            self._terminal_result = ProcessResult(
+                self._terminal.returncode, stdout, stderr, cmdline=self._terminal.args
+            )
             return self._terminal_result
         finally:
             self._terminal = None
@@ -294,48 +305,57 @@ class FactoryScriptBase(FactoryProcess):
         """
         Run the given command synchronously
         """
+        start_time = time.time()
         timeout = kwargs.pop("_timeout", None) or self.default_timeout
-        timeout_expire = time.time() + timeout
 
         # Build the cmdline to pass to the terminal
         # We set the _terminal_timeout attribute while calling build_cmdline in case it needs
         # access to that information to build the command line
         try:
             self._terminal_timeout = timeout
-            proc_args = self.build_cmdline(*args, **kwargs)
+            cmdline = self.build_cmdline(*args, **kwargs)
+            timeout_expire = time.time() + self._terminal_timeout
+
+            log.info("%sRunning %r in CWD: %s ...", self.get_log_prefix(), cmdline, self.cwd)
+
+            terminal = self.init_terminal(cmdline, cwd=self.cwd, env=self.environ,)
+            timmed_out = False
+            while True:
+                if timeout_expire < time.time():
+                    timmed_out = True
+                    break
+                if terminal.poll() is not None:
+                    break
+                time.sleep(0.25)
+
+            result = self.terminate()
+            if timmed_out:
+                raise ProcessTimeout(
+                    "{}Failed to run: {}; Error: Timed out after {} seconds!".format(
+                        self.get_log_prefix(), cmdline, timeout
+                    ),
+                    stdout=result.stdout,
+                    stderr=result.stderr,
+                    cmdline=cmdline,
+                )
+
+            exitcode = result.exitcode
+            stdout, stderr, json_out = self.process_output(
+                result.stdout, result.stderr, cmdline=cmdline
+            )
+            log.info(
+                "%sCompleted %r in CWD: %s after %.2f seconds",
+                self.get_log_prefix(),
+                cmdline,
+                self.cwd,
+                time.time() - start_time,
+            )
+            return ShellResult(exitcode, stdout, stderr, json=json_out, cmdline=cmdline)
         finally:
             # We now clear the _terminal_timeout attribute
             self._terminal_timeout = None
 
-        log.info("%sRunning %r in CWD: %s ...", self.get_log_prefix(), proc_args, self.cwd)
-
-        terminal = self.init_terminal(proc_args, cwd=self.cwd, env=self.environ,)
-        timmed_out = False
-        while True:
-            if timeout_expire < time.time():
-                timmed_out = True
-                break
-            if terminal.poll() is not None:
-                break
-            time.sleep(0.25)
-
-        result = self.terminate()
-        if timmed_out:
-            raise ProcessTimeout(
-                "{}Failed to run: {}; Error: Timed out after {} seconds!".format(
-                    self.get_log_prefix(), proc_args, timeout
-                ),
-                stdout=result.stdout,
-                stderr=result.stderr,
-            )
-
-        exitcode = result.exitcode
-        stdout, stderr, json_out = self.process_output(
-            result.stdout, result.stderr, cli_cmd=proc_args
-        )
-        return ShellResult(exitcode, stdout, stderr, json_out)
-
-    def process_output(self, stdout, stderr, cli_cmd=None):
+    def process_output(self, stdout, stderr, cmdline=None):
         if stdout:
             try:
                 json_out = json.loads(stdout)
@@ -362,10 +382,10 @@ class FactoryPythonScriptBase(FactoryScriptBase):
         self.environ.setdefault(str("PYTHONDONTWRITEBYTECODE"), str("1"))
 
     def build_cmdline(self, *args, **kwargs):
-        proc_args = super(FactoryPythonScriptBase, self).build_cmdline(*args, **kwargs)
-        if proc_args[0] != self.python_executable:
-            proc_args.insert(0, self.python_executable)
-        return proc_args
+        cmdline = super(FactoryPythonScriptBase, self).build_cmdline(*args, **kwargs)
+        if cmdline[0] != self.python_executable:
+            cmdline.insert(0, self.python_executable)
+        return cmdline
 
 
 class FactoryDaemonScriptBase(FactoryProcess):
