@@ -18,6 +18,7 @@ import time
 import weakref
 
 import psutil
+import six
 
 from saltfactories.exceptions import ProcessNotStarted
 from saltfactories.utils import ports
@@ -237,10 +238,7 @@ def start_daemon(
     attempts = 0
     log_prefix = ""
 
-    if sys.platform.startswith("win"):
-        # Double the start timeout on windows
-        start_timeout = start_timeout * 2
-
+    checks_start_time = time.time()
     while attempts <= max_attempts:  # pylint: disable=too-many-nested-blocks
         attempts += 1
         process = daemon_class(cli_script_name=cli_script_name, **extra_daemon_class_kwargs)
@@ -286,13 +284,16 @@ def start_daemon(
                     result = process.terminate()
                     if attempts >= max_attempts:
                         raise ProcessNotStarted(
-                            "{}The {!r} has failed to confirm running status after {} attempts".format(
-                                log_prefix, process, attempts
+                            "{}The {!r} has failed to confirm running status after {} attempts, which "
+                            "took {:.2f} seconds".format(
+                                log_prefix, process, attempts, time.time() - checks_start_time
                             ),
                             stdout=result.stdout,
                             stderr=result.stderr,
                         )
                     continue
+            except ProcessNotStarted:
+                six.reraise(*sys.exc_info())
             except Exception as exc:  # pylint: disable=broad-except
                 log.exception(
                     "%sException caugth on %r: %s", log_prefix, process, exc, exc_info=True
@@ -301,7 +302,9 @@ def start_daemon(
                 if attempts >= max_attempts:
                     raise ProcessNotStarted(
                         "{}The {!r} has failed to confirm running status after {} attempts and raised an "
-                        "exception: {}.".format(log_prefix, process, attempts, str(exc)),
+                        "exception: {}. Took {:.2f} seconds.".format(
+                            log_prefix, process, attempts, str(exc), time.time() - start_time
+                        ),
                         stdout=result.stdout,
                         stderr=result.stderr,
                         exc=sys.exc_info(),
@@ -309,7 +312,13 @@ def start_daemon(
                 continue
             # A little breathing before returning the process
             time.sleep(0.125)
-            log.info("%sThe %r is running after %d attempts", log_prefix, process, attempts)
+            log.info(
+                "%sThe %r is running after %d attempts. Took %1.2f seconds",
+                log_prefix,
+                process,
+                attempts,
+                time.time() - checks_start_time,
+            )
             break
         else:
             process.terminate()
@@ -317,18 +326,17 @@ def start_daemon(
             time.sleep(1)
             continue
     else:
+        stderr = stdout = None
         if process is not None:
             result = process.terminate()
-            raise ProcessNotStarted(
-                "{}The {!r} has failed to confirm running status after {} attempts.".format(
-                    log_prefix, process, attempts
-                ),
-                stdout=result.stdout,
-                stderr=result.stderr,
-            )
+            stderr = result.stderr
+            stdout = result.stdout
         raise ProcessNotStarted(
-            "{}The {!r} has failed to confirm running status after {} attempts.".format(
-                log_prefix, process, attempts
+            "{}The {!r} has failed to confirm running status after {} attempts, which "
+            "took {:.2f} seconds.".format(
+                log_prefix, process, attempts, time.time() - checks_start_time
             ),
+            stdout=stdout,
+            stderr=stderr,
         )
     return process
