@@ -13,6 +13,22 @@ import logging
 import os
 import sys
 
+import six
+
+try:
+    from salt.utils.parsers import SaltKeyOptionParser
+
+    try:
+        SALT_KEY_LOG_LEVEL_SUPPORTED = SaltKeyOptionParser._skip_console_logging_config_ is False
+    except AttributeError:
+        # New logging is in place
+        SALT_KEY_LOG_LEVEL_SUPPORTED = True
+except ImportError:  # pragma: no cover
+    # We need salt to test salt with saltfactories, and, when pytest is rewriting modules for proper assertion
+    # reporting, we still haven't had a chance to inject the salt path into sys.modules, so we'll hit this
+    # import error, but its safe to pass
+    SALT_KEY_LOG_LEVEL_SUPPORTED = False
+
 from saltfactories.utils.processes.bases import FactoryDaemonScriptBase
 from saltfactories.utils.processes.bases import FactoryPythonScriptBase
 
@@ -37,6 +53,7 @@ class SaltConfigMixin(object):
 class SaltScriptBase(FactoryPythonScriptBase, SaltConfigMixin):
 
     __cli_timeout_supported__ = False
+    __cli_log_level_supported__ = True
 
     def __init__(self, *args, **kwargs):
         config = kwargs.pop("config", None) or {}
@@ -50,8 +67,6 @@ class SaltScriptBase(FactoryPythonScriptBase, SaltConfigMixin):
         config_dir = self.config_dir
         if config_dir:
             script_args.append("--config-dir={}".format(config_dir))
-        script_args.append("--log-level=quiet")
-        script_args.append("--out=json")
         return script_args
 
     def get_script_args(self):
@@ -71,15 +86,34 @@ class SaltScriptBase(FactoryPythonScriptBase, SaltConfigMixin):
     def build_cmdline(self, *args, **kwargs):  # pylint: disable=arguments-differ
         log.debug("Building cmdline. Input args: %s; Input kwargs: %s;", args, kwargs)
         minion_tgt = self._minion_tgt = self.get_minion_tgt(kwargs)
-        proc_args = []
-        if minion_tgt:
-            proc_args.append(minion_tgt)
+        cmdline = []
 
+        args = list(args)
+
+        # Handle the timeout CLI flag, if supported
         if self.__cli_timeout_supported__:
-            args = list(args)
+            salt_cli_timeout_next = False
             for arg in args:
-                if arg == "-t" or arg.startswith("--timeout"):
+                if arg.startswith("--timeout="):
+                    # Let's actually change the _terminal_timeout value which is used to
+                    # calculate when the run() method should actually timeout
+                    salt_cli_timeout = arg.split("--timeout=")[-1]
+                    try:
+                        self._terminal_timeout = int(salt_cli_timeout) + 5
+                    except ValueError:
+                        # Not a number? Let salt do it's error handling
+                        pass
                     break
+                if salt_cli_timeout_next:
+                    try:
+                        self._terminal_timeout = int(arg) + 5
+                    except ValueError:
+                        # Not a number? Let salt do it's error handling
+                        pass
+                    break
+                if arg == "-t" or arg.startswith("--timeout"):
+                    salt_cli_timeout_next = True
+                    continue
             else:
                 salt_cli_timeout = self._terminal_timeout
                 if salt_cli_timeout:
@@ -89,12 +123,52 @@ class SaltScriptBase(FactoryPythonScriptBase, SaltConfigMixin):
                     # If it's still a positive number, add it to the salt command CLI flags
                     args.append("--timeout={}".format(salt_cli_timeout))
 
-        proc_args += list(args)
+        # Handle the output flag
+        for arg in args:
+            if arg in ("--out", "--output"):
+                break
+            if arg.startswith(("--out=", "--output=")):
+                break
+        else:
+            # No output was passed, the default output is JSON
+            cmdline.append("--out=json")
+
+        if self.__cli_log_level_supported__:
+            # Handle the logging flag
+            for arg in args:
+                if arg in ("-l", "--log-level"):
+                    break
+                if arg.startswith("--log-level="):
+                    break
+            else:
+                # Default to being quiet on console output
+                cmdline.append("--log-level=quiet")
+
+        if minion_tgt:
+            cmdline.append(minion_tgt)
+
+        # Add the remaning args
+        cmdline.extend(args)
+
         for key in kwargs:
-            proc_args.append("{}={}".format(key, kwargs[key]))
-        proc_args = super(SaltScriptBase, self).build_cmdline(*proc_args)
-        log.debug("Built cmdline: %s", proc_args)
-        return proc_args
+            cmdline.append("{}={}".format(key, kwargs[key]))
+        cmdline = super(SaltScriptBase, self).build_cmdline(*cmdline)
+        log.debug("Built cmdline: %s", cmdline)
+        return cmdline
+
+    def process_output(self, stdout, stderr, cmdline=None):
+        stdout, stderr, json_out = super(SaltScriptBase, self).process_output(
+            stdout, stderr, cmdline=cmdline
+        )
+        if json_out and isinstance(json_out, six.string_types) and "--out=json" in cmdline:
+            # Sometimes the parsed JSON is just a string, for example:
+            #  OUTPUT: '"The salt master could not be contacted. Is master running?"\n'
+            #  LOADED JSON: 'The salt master could not be contacted. Is master running?'
+            #
+            # In this case, we assign the loaded JSON to stdout and reset json_out
+            stdout = json_out
+            json_out = None
+        return stdout, stderr, json_out
 
 
 class SaltDaemonScriptBase(FactoryDaemonScriptBase, FactoryPythonScriptBase, SaltConfigMixin):
@@ -219,14 +293,14 @@ class SaltCLI(SaltScriptBase):
 
     __cli_timeout_supported__ = True
 
-    def process_output(self, stdout, stderr, cli_cmd=None):
+    def process_output(self, stdout, stderr, cmdline=None):
         if "No minions matched the target. No command was sent, no jid was assigned.\n" in stdout:
             stdout = stdout.split("\n", 1)[1:][0]
         old_stdout = None
-        if "--show-jid" in cli_cmd and stdout.startswith("jid: "):
+        if "--show-jid" in cmdline and stdout.startswith("jid: "):
             old_stdout = stdout
             stdout = stdout.split("\n", 1)[-1].strip()
-        stdout, stderr, json_out = SaltScriptBase.process_output(self, stdout, stderr, cli_cmd)
+        stdout, stderr, json_out = SaltScriptBase.process_output(self, stdout, stderr, cmdline)
         if old_stdout is not None:
             stdout = old_stdout
         if json_out:
@@ -250,10 +324,10 @@ class SaltCallCLI(SaltScriptBase):
     def get_minion_tgt(self, kwargs):
         return None
 
-    def process_output(self, stdout, stderr, cli_cmd=None):
+    def process_output(self, stdout, stderr, cmdline=None):
         # Under salt-call, the minion target is always "local"
         self._minion_tgt = "local"
-        stdout, stderr, json_out = SaltScriptBase.process_output(self, stdout, stderr, cli_cmd)
+        stdout, stderr, json_out = SaltScriptBase.process_output(self, stdout, stderr, cmdline)
         if json_out:
             if not isinstance(json_out, dict):
                 # A string was most likely loaded, not what we want.
@@ -283,10 +357,10 @@ class SaltCpCLI(SaltScriptBase):
 
     __cli_timeout_supported__ = True
 
-    def process_output(self, stdout, stderr, cli_cmd=None):
+    def process_output(self, stdout, stderr, cmdline=None):
         if "No minions matched the target. No command was sent, no jid was assigned.\n" in stdout:
             stdout = stdout.split("\n", 1)[1:][0]
-        stdout, stderr, json_out = SaltScriptBase.process_output(self, stdout, stderr, cli_cmd)
+        stdout, stderr, json_out = SaltScriptBase.process_output(self, stdout, stderr, cmdline)
         if json_out:
             if not isinstance(json_out, dict):
                 # A string was most likely loaded, not what we want.
@@ -303,15 +377,9 @@ class SaltKeyCLI(SaltScriptBase):
     Simple subclass to the salt-key CLI script
     """
 
-    def get_base_script_args(self):
-        script_args = super(SaltKeyCLI, self).get_base_script_args()
-        # As of Neon, salt-key still does not support --log-level
-        # Only when we get the new logging merged in will we get that, so remove that CLI flag
-        for idx, flag in enumerate(script_args):
-            if flag.startswith("--log-level="):
-                script_args.pop(idx)
-                break
-        return script_args
+    # As of Neon, salt-key still does not support --log-level
+    # Only when we get the new logging merged in will we get that, so remove that CLI flag
+    __cli_log_level_supported__ = SALT_KEY_LOG_LEVEL_SUPPORTED
 
     def get_minion_tgt(self, kwargs):
         return None
